Patterns for Loading — stevedore 0.11.a6.gf4c43cc documentation

### Navigation

- index
- next |
- previous |
- stevedore 0.11.a6.gf4c43cc documentation »

# Patterns for Loading

Setuptools entry points are registered as within a namespace that defines the API expected by the plugin code. Each entry point has a name, which does not have to be unique within a given namespace. The flexibility of this name management system makes it possible to use plugins in a variety of ways. The manager classes in stevedore wrap `pkg_resources` to apply different rules matching the patterns described here.

## Drivers – Single Name, Single Entry Point

Specifying a driver for communicating with an external resource (database, device, or remote application) is perhaps the most common use of dynamically loaded libraries. Drivers support the abstracted view of the resource so an application can work with different types of resources. For example, drivers may connect to database engines, load different file formats, or communicate with similar web services from different providers. Many drivers may be available for a given application, but it is implied in the interface between the application and the driver that only one driver will be used to manage a given resource.

Examples of the drivers pattern include:

- database client libraries used by SQLAlchemy
- cloud vendor API clients used by libcloud

See also

`stevedore.driver.DriverManager`

## Hooks – Single Name, Many Entry Points

Hooks, signals, or callbacks are invoked based on an event occuring within an application. All of the hooks for an application may share a single namespace (e.g., `my.application.hooks`) and use a different name for the triggered event (e.g., `startup` and `precommit`). Multiple entry points can share the same name within the namespace, so that multiple hooks can be invoked when an event occurs.

Examples of the hooks pattern include:

- Emacs mode hook functions
- Django signals

See also

`stevedore.hook.HookManager`

## Extensions – Many Names, Many Entry Points

The more general form of extending an application is to load additional functionality by discovering add-on modules that use a minimal API to inject themselves at runtime. Extensions typically want to be notified that they have been loaded and are being used so they can perform initialization or setup steps. An extension may replace core functionality or add to it.

Examples of the extensions pattern include:

- Django apps
- Sphinx extensions
- Trac Plugins

See also

`stevedore.extension.ExtensionManager`

### Navigation

- index
- next |
- previous |
- stevedore 0.11.a6.gf4c43cc documentation »